use egui::{Color32, CursorIcon, ImageSource, RichText, Sense, Vec2};
use xmpd_cache::DlStatus;
use xmpd_manifest::{query, song::Song, store::BaseStore};
use crate::utils::SearchType;

use super::{CompGetter, CompUi};

pub mod header;

#[derive(Debug, Default)]
pub struct SongList {
    pub selected_sid: uuid::Uuid,
    playable_songs: Vec<uuid::Uuid>,
}

component_register!(SongList);

impl CompUi for SongList {
    fn draw(ui: &mut egui::Ui, state: &mut crate::GuiState) -> crate::Result<()> {
        let songs = Self::get_and_sort_songs(state)?;
        let disp_songs = Self::get_songs_to_display(&songs)?;
        {
            let mut sl = SongList::get()?;
            sl.playable_songs = Self::get_playable_songs(&songs)?;
            if let Some((sid, _)) = songs.first() {
                if sl.selected_sid == Default::default() {
                    sl.selected_sid = sid.clone();
                }
            }
        }
        egui::ScrollArea::vertical()
            .id_source("song_list")
            .drag_to_scroll(false)
            .show(ui, |ui| {
                ui.vertical(|ui| {
                    ui.add_space(3.0);
                    for (sid, song) in disp_songs {
                         handle_error_ui!(Self::display_song_tab(ui, state, &sid, &song));
                    }
                });
        });
        Ok(())
    }
}

impl SongList {
    fn get_and_sort_songs(state: &mut crate::GuiState) -> crate::Result<Vec<(uuid::Uuid, Song)>> {
        let pid = super::left_nav::LeftNav::get()?.selected_playlist_id.clone();
        match pid {
            None => {
                let songs = state.manifest.store().get_songs().clone().into_iter();
                let mut songs: Vec<_> = songs.collect();
                songs.sort_by(|a, b| {
                    let a = a.1.name().to_lowercase();
                    let b = b.1.name().to_lowercase();
                    a.cmp(&b)
                });
                Ok(songs)
            }
            Some(pid) => {
                let Some(playlist) = state.manifest.store().get_playlist(&pid) else {
                    anyhow::bail!("Couldnt find playlist (corruption?)");
                };
                let mut songs = Vec::new();
                for sid in playlist.songs() {
                    if let Some(song) = state.manifest.store().get_song(&sid) {
                        songs.push((sid.clone(), song.clone()));
                    }
                }
                songs.sort_by(|a, b| {
                    let a = a.1.name().to_lowercase();
                    let b = b.1.name().to_lowercase();
                    a.cmp(&b)
                });
                Ok(songs)
            }
        }
    }

    fn get_songs_to_display(songs: &Vec<(uuid::Uuid, Song)>) -> crate::Result<Vec<(uuid::Uuid, Song)>>{
        let mut to_display = Vec::new();
        let query = {header::Header::get()?.search_text.clone()};
        let (query_type, query_text) = crate::utils::SearchType::from_str(&query);
        for (sid, song) in songs {
            let should_display = match &query_type {
                SearchType::Normal | 
                SearchType::Author |
                SearchType::Source if query_text.is_empty() => true,

                SearchType::Source => {
                    song.source_type().to_string()
                        .to_lowercase()
                        .contains(&query_text)
                },
                SearchType::Author => {
                    song.author()
                        .to_lowercase()
                        .contains(&query_text)
                },
                SearchType::Normal => {
                    song.name()
                        .to_lowercase()
                        .contains(&query_text)
                },
            };

            if should_display {
                to_display.push((sid.clone(), song.clone()));
            }
        }
        Ok(to_display)
    }

    fn display_song_tab(ui: &mut egui::Ui, state: &mut crate::GuiState, sid: &uuid::Uuid, song: &Song) -> crate::Result<()> {
        let mut clicked = false;
        ui.horizontal(|ui| {
            let theme = handle_error_ui!(xmpd_settings::Settings::get()).theme.clone();
            // let icon_status = handle_error_ui!(xmpd_cache::Cache::get()).get_cached_icon_status(&sid).clone();
            let img = ui.add(
                //if let Some(DlStatus::Done(Some(p))) = icon_status {
                //    let uri: Cow<str> = Cow::Owned(p.to_string_lossy().to_string());
                //    let bytes = handle_error_ui!(std::fs::read(p));
                //    ui.ctx().include_bytes(uri.clone(), bytes);
                //    egui::Image::new(ImageSource::Uri(uri))
                //        .sense(Sense::click())
                //        .fit_to_exact_size(Vec2::new(32.0, 32.0))
                // } else {
                    egui::Image::new(crate::data::NOTE_ICON)
                        .tint(theme.accent_color)
                        .sense(Sense::click())
                        .fit_to_exact_size(Vec2::new(32.0, 32.0))
                //}
            );
            let status = {
                handle_error_ui!(xmpd_cache::Cache::get()).get_cached_song_status(&sid).clone()
            };
            if img.clicked() {
                clicked = true;
            }
            if img.hovered() {
                if matches!(status, Some(DlStatus::Done(_))) {
                    ui.output_mut(|o| o.cursor_icon = CursorIcon::PointingHand);
                } else {
                    ui.output_mut(|o| o.cursor_icon = CursorIcon::Default);
                }
            } 
            // img.context_menu(|ui| handle_error_ui!(Self::show_context_menu(ui, sid, song)));

            ui.vertical(|ui| {
                let slf = handle_error_ui!(SongList::get());
                let label = if slf.selected_sid == *sid {
                    RichText::new(song.name())
                        .color(theme.accent_color)
                } else if matches!(status, Some(DlStatus::Done(_))) {
                    RichText::new(song.name())
                        .color(theme.text_color)
                } else {
                    RichText::new(song.name())
                        .color(theme.dim_text_color)
                };
                let label = ui.label(label);

                if label.clicked() {
                    clicked = true;
                }
                if label.hovered() {
                    if matches!(status, Some(DlStatus::Done(_))) {
                        ui.output_mut(|o| o.cursor_icon = CursorIcon::PointingHand);
                    } else {
                        ui.output_mut(|o| o.cursor_icon = CursorIcon::Default);
                    }
                }

                // label.context_menu(|ui| handle_error_ui!(Self::show_context_menu(ui, sid, song)));
                ui.monospace(
                    RichText::new(format!("By {}", song.author()))
                        .color(theme.dim_text_color)
                        .size(10.0)
                );

            });
            
            ui.with_layout(egui::Layout::right_to_left(egui::Align::RIGHT), |ui| {
                ui.add_space(3.0);
                
                match status {
                    Some(DlStatus::Done(_)) => {
                        //let img = egui::Image::new(crate::data::CHECK_ICON)
                        //    .tint(Color32::LIGHT_GREEN)
                        //    .sense(Sense::hover())
                        //    .fit_to_exact_size(Vec2::new(16.0, 16.0));

                        //ui.add(img).on_hover_ui(|ui| {
                        //    ui.label(format!("Path: {p}"));
                        //});
                    }
                    Some(DlStatus::Downloading) => {
                        let spinner = egui::Spinner::new()
                                .color(theme.accent_color)
                                .size(16.0);
                        ui.add(spinner);
                    }
                    Some(DlStatus::Error(e, ..)) => {
                        let img = egui::Image::new(crate::data::WARN_ICON)
                                .tint(Color32::LIGHT_YELLOW)
                                .sense(Sense::hover())
                                .fit_to_exact_size(Vec2::new(16.0, 16.0));

                        ui.add(img).on_hover_ui(|ui| {
                            ui.label(e);
                        });
                    }
                    None => {
                        let img = egui::Image::new(crate::data::DL_ICON)
                                .tint(theme.accent_color)
                                .sense(Sense::click())
                                .fit_to_exact_size(Vec2::new(16.0, 16.0));

                        if ui.add(img).clicked() {
                            handle_error_ui!(xmpd_cache::Cache::get()).download_song_to_cache(sid.clone(), song.clone());
                            let mut toast = handle_error_ui!(crate::components::toast::Toast::get());
                            toast.show_toast(
                                "Downloading Song",
                                &format!("Started downloading {} by {}", song.name(), song.author()), 
                                super::toast::ToastType::Info
                            );
                        }
                    }
                }
            });    
        });

        ui.separator();
        if clicked {
            let mut sl = SongList::get()?;
            sl.play_song(sid.clone(), state)?;
        }

        Ok(())
    }
    pub fn play_song(&mut self, sid: uuid::Uuid, state: &mut crate::GuiState) -> crate::Result<()> {
        if self.playable_songs.contains(&sid) {
            self.selected_sid = sid.clone();
            let path = state.manifest.get_song_as_path(sid)?;
            state.player.play_song(&path)?;
        }
        Ok(())
    }
    pub fn play_prev(&mut self, state: &mut crate::GuiState) -> crate::Result<()> {
        let Some(mut prev) = self.playable_songs.last().cloned() else {
            anyhow::bail!("Trying to play a song in an empty playlist (impossible)")
        };
        for sid in self.playable_songs.clone() {
            if sid == self.selected_sid {
                self.play_song(prev, state)?;
            }
            prev = sid;
        }
        Ok(())
    }
    pub fn play_next(&mut self, state: &mut crate::GuiState) -> crate::Result<()> {
        let Some(mut next) = self.playable_songs.first().cloned() else {
            anyhow::bail!("Trying to play a song in an empty playlist (impossible)")
        };
        let mut found = false;
        for sid in self.playable_songs.clone() {
            if sid == self.selected_sid {
                found = true;
            } else if found {
                next = sid;
                break;
            } 
        }
        self.play_song(next, state)?;
        Ok(())
    }
    fn get_playable_songs(songs: &Vec<(uuid::Uuid, Song)>) -> crate::Result<Vec<uuid::Uuid>> {
        let mut playable_songs = Vec::new();
        
        for (sid, _) in songs {
            if let Some(DlStatus::Done(_)) = xmpd_cache::Cache::get()?.get_cached_song_status(&sid) {
                playable_songs.push(sid.clone());
            }
        }
        Ok(playable_songs)
    }
    //fn show_context_menu(ui: &mut egui::Ui, sid: &uuid::Uuid, song: &Song) -> crate::Result<()> {
    //    if ui.button("Download icon").clicked() {
    //        xmpd_cache::Cache::get()?.download_icon_to_cache(sid.clone(), song.clone());
    //    }
    //    Ok(())
    //}
}
